#include "HUD.h"

HUD::HUD(GLint width, GLint height) {
	this->width = width;
	this->height = height;
}

HUD::~HUD(void) {
}

void HUD::drawHUD() {
	beginHUD();
	drawLabels();
	drawButtons();
	endHUD();
}

void HUD::addLabel(GLint x, GLint y, const char *string, GLfloat fontColor[4], void *font) {
	Label *lb = new Label;

	lb->x = x;
	lb->y = y;
	lb->name = string;

	// set fonts
	if (font == NULL)  {
		lb->font = GLUT_BITMAP_HELVETICA_18;
	}
	else {
		lb->font = font;
	}

	// set color
	if (fontColor == NULL)  {
		lb->fontColor[0] = 1.0f;
		lb->fontColor[1] = 1.0f;
		lb->fontColor[2] = 1.0f;
		lb->fontColor[3] = 1.0f;
	}
	else {
		lb->fontColor[0] = fontColor[0];
		lb->fontColor[1] = fontColor[1];
		lb->fontColor[2] = fontColor[2];
		lb->fontColor[3] = fontColor[3];
	}

	// finally put label into vector
	labels.push_back(lb);
}

void HUD::addButton(GLint x, GLint y, GLint dx, GLint dy, int ID, const char *string, const char *picture_path, GLfloat fontColor[4], void *font) {
	Button *btn = new Button;

	btn->x = x;			btn->y = y;
	btn->dx = dx;		btn->dy = dy;
	btn->ID = ID;
	btn->name = string;

	// set fonts
	if (font == NULL)  {
		btn->font = GLUT_BITMAP_HELVETICA_18;
	}
	else {
		btn->font = font;
	}

	// set color
	if (fontColor == NULL)  {
		btn->fontColor[0] = 1.0f;
		btn->fontColor[1] = 1.0f;
		btn->fontColor[2] = 1.0f;
		btn->fontColor[3] = 1.0f;
	}
	else {
		btn->fontColor[0] = fontColor[0];
		btn->fontColor[1] = fontColor[1];
		btn->fontColor[2] = fontColor[2];
		btn->fontColor[3] = fontColor[3];
	}

	if (picture_path != NULL) {
		// Load Texture
		glTexture *tex = new glTexture;
		TextureLoader loader;
		loader.SetTextureFilter(txBilinear);
		//loader.SetMipMapping(true);
		loader.LoadTextureFromDisk((char*)picture_path, tex);

		btn->tex = tex;
	}
	else {
		btn->tex = NULL;
	}

	// finally put label into vector
	buttons.push_back(btn);
}

void HUD::callback(void func(int)) {
	callback_func = func;
}

int HUD::mouse(int button, int state, int x, int y) {
	y = height - y;

	/* traverse vector draw Buttons */
	for(vector<Button*>::iterator iter = buttons.begin(); iter != buttons.end(); ++iter) {
		Button *btn = *iter;

		if ((btn->x < x) && (btn->y < y) && (btn->x + btn->dx > x) && (btn->y + btn->dy > y)) {
			callback_func(btn->ID);
			return 1;
		}
	}

	return 0;
}

/* 
 * Place for private method/functions
 */
void HUD::beginHUD(void) {
	// Disables Depth Testing
	glDisable(GL_DEPTH_TEST);

	// Select The Projection Matrix and store it
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();

	// Reset The Projection Matrix
	glLoadIdentity();
	glOrtho(0,width,0,height,-1,1);

	// Select The Modelview Matrix
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	// set some settings and save their status
	glGetBooleanv(GL_TEXTURE_2D, &is_texuture_2d_enable);
//	glGetBooleanv(GL_COLOR_MATERIAL, &is_color_material_enable);

	// disable unneeded settings
	glDisable(GL_TEXTURE_2D);
//	glEnable(GL_COLOR_MATERIAL);

	// Push Attributes
	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
}

void HUD::endHUD(void) {
	// Pop Attributes
	glPopAttrib();
	glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH_TEST);

	// set previous values of those settings
	if (is_texuture_2d_enable)
		glEnable(GL_TEXTURE_2D);

//	if (!is_color_material_enable)
//		glDisable(GL_COLOR_MATERIAL);

	// undo changes
	glPopMatrix();										// Restore The Old Modelview Matrix
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
}

void HUD::drawLabels(void) {
	glPushMatrix();

	/* traverse vector draw Label */
	for(vector<Label*>::iterator iter = labels.begin(); iter != labels.end(); ++iter) {
		Label *lb = *iter;

		glColor4fv(lb->fontColor);
		glRasterPos2i(lb->x, lb->y);

		const char* p;
		for (p = lb->name; *p; p++) {
			glutBitmapCharacter(lb->font, *p);
		}
	}

	glPopMatrix();
}

void HUD::drawButtons(void) {
	/* traverse vector draw Buttons */
	for(vector<Button*>::iterator iter = buttons.begin(); iter != buttons.end(); ++iter) {
		Button *btn = *iter;

		glColor4fv(btn->fontColor);
		glRasterPos2i(btn->x, btn->y);

		if (btn->tex == NULL) {
			const char* p;
			for (p = btn->name; *p; p++) {
				glutBitmapCharacter(btn->font, *p);
			}
		}
		else {
			// clip texture
			glEnable(GL_TEXTURE_2D);
glPushMatrix();

			glTranslatef(btn->x, btn->y, 0.0f);
			glBindTexture(GL_TEXTURE_2D, btn->tex->TextureID);

			glBegin(GL_QUADS);
				glTexCoord2f(0.0f, 0.0f);		glVertex2f(0.0f, 0.0f);
				glTexCoord2f(1.0f, 0.0f);		glVertex2f(btn->dx, 0.0f);
				glTexCoord2f(1.0f, 1.0f);		glVertex2f(btn->dx, btn->dy);
				glTexCoord2f(0.0f, 1.0f);		glVertex2f(0.0f, btn->dy);
			glEnd();
glPopMatrix();

			glDisable(GL_TEXTURE_2D);
		}
	}
}
